/* --------------------------------------------------------------------
Copyright (C) 2009-2010 Swedish Meteorological and Hydrological Institute, SMHI,

This file is part of the beast-ui package.

The beast-ui package is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

The beast-ui package is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with the beast-ui package library.  If not, see <http://www.gnu.org/licenses/>.
------------------------------------------------------------------------*/
package eu.baltrad.beastui.web.controller;

import java.util.ArrayList;
import java.util.List;

import junit.framework.TestCase;

import org.easymock.MockControl;
import org.easymock.classextension.MockClassControl;
import org.springframework.ui.Model;

import eu.baltrad.beast.adaptor.IBltAdaptorManager;
import eu.baltrad.beast.router.IRouterManager;
import eu.baltrad.beast.router.RouteDefinition;
import eu.baltrad.beast.rules.RuleException;
import eu.baltrad.beast.rules.groovy.GroovyRule;

/**
 * @author Anders Henja
 */
public class GroovyRoutesControllerTest extends TestCase {
  private static interface MethodMocker {
    public String viewCreateRoute(Model model,
        String name,
        String author,
        Boolean active,
        String description,
        List<String> recipients,
        String definition,
        String emessage);
    
    public String viewShowRoutes(Model model,
        String emessage);
    
    public String viewShowRoute(Model model,
        String name,
        String author,
        Boolean active,
        String description,
        List<String> recipients,
        String definition,
        String emessage); 
    
    public GroovyRule createRule(String script);
    
    public String modifyRoute(Model model, String name, String author,
        Boolean active, String description, List<String> recipients, String script);
  };
  private GroovyRoutesController classUnderTest = null;
  private MockControl managerControl = null;
  private IRouterManager manager = null;
  private MockControl adaptorControl = null;
  private IBltAdaptorManager adaptorManager = null;
  private MockControl modelControl = null;
  private Model model = null;
  private MockControl methodMockControl = null;
  private MethodMocker methodMock = null;
  
  public void setUp() throws Exception {
    managerControl = MockControl.createControl(IRouterManager.class);
    manager = (IRouterManager) managerControl.getMock();
    adaptorControl = MockControl.createControl(IBltAdaptorManager.class);
    adaptorManager = (IBltAdaptorManager) adaptorControl.getMock();
    modelControl = MockControl.createControl(Model.class);
    model = (Model) modelControl.getMock();
    methodMockControl = MockControl.createControl(MethodMocker.class);
    methodMock = (MethodMocker)methodMockControl.getMock();
    
    // All creator methods and view methods are mocked to simplify testing
    //
    classUnderTest = new GroovyRoutesController() {
      protected String viewCreateRoute(Model model,
          String name,
          String author,
          Boolean active,
          String description,
          List<String> recipients,
          String definition,
          String emessage) {
        return methodMock.viewCreateRoute(model, name, author, active, description, recipients, definition, emessage);
      }
      
      protected String viewShowRoutes(Model model,
          String emessage) {
        return methodMock.viewShowRoutes(model, emessage);
      }
      
      protected String viewShowRoute(Model model,
          String name,
          String author,
          Boolean active,
          String description,
          List<String> recipients,
          String definition,
          String emessage) {
        return methodMock.viewShowRoute(model, name, author, active,
            description, recipients, definition, emessage);
      }
      
      protected GroovyRule createRule(String script) {
        return methodMock.createRule(script);
      }
      
      protected String modifyRoute(Model model, String name, String author,
          Boolean active, String description, List<String> recipients, String script) {
        return methodMock.modifyRoute(model, name, author, active, description, recipients, script);
      }
    };
    classUnderTest.setManager(manager);
    classUnderTest.setAdaptorManager(adaptorManager);
  }

  public void tearDown() throws Exception {
    classUnderTest = null;
    managerControl = null;
    manager = null;
    adaptorControl = null;
    adaptorManager = null;
    modelControl = null;
    model = null;
    methodMockControl = null;
    methodMock = null;
  }

  private void replay() {
    managerControl.replay();
    adaptorControl.replay();
    modelControl.replay();
    methodMockControl.replay();
  }

  private void verify() {
    managerControl.verify();
    adaptorControl.verify();
    modelControl.verify();
    methodMockControl.verify();
  }

  public void testCreateRoute_initial() {
    List<String> types = new ArrayList<String>();
    types.add("groovy");
    methodMock.viewCreateRoute(model, null, null, null, null, null, null,null);
    methodMockControl.setReturnValue("route_create_groovy");
    
    replay();

    String result = classUnderTest.createRoute(model, null, null, null,
        null, null, null);

    verify();
    assertEquals("route_create_groovy", result);
  }

  public void testCreateRoute_noAdaptors() {
    methodMock.viewCreateRoute(model, null, null, null, null, null, null,null);
    methodMockControl.setReturnValue("route_create_groovy");

    replay();

    String result = classUnderTest.createRoute(model, null, null, null,
        null, null, null);

    verify();
    assertEquals("route_create_groovy", result);
  }

  public void testCreateRoute_success() {
    RouteDefinition routedef = new RouteDefinition();
    String name = "MyRoute";
    String author = "nisse";
    Boolean active = true;
    String description = "some description";
    String def = "some code";
    // GroovyRule constructor is protected so mock it
    MockControl ruleControl = MockClassControl.createControl(GroovyRule.class);
    GroovyRule rule = (GroovyRule)ruleControl.getMock();
    
    List<String> recipients = new ArrayList<String>();
    methodMock.createRule(def);
    methodMockControl.setReturnValue(rule);
    manager.create(name, author, active, description, recipients, rule);
    managerControl.setReturnValue(routedef);
    manager.storeDefinition(routedef);

    replay();

    String result = classUnderTest.createRoute(model, name, author, active,
        description, recipients, def);

    verify();
    assertEquals("redirect:routes.htm", result);
  }

  public void testCreateRoute_noName() {
    String name = null;
    String author = "nisse";
    Boolean active = true;
    String description = "some description";
    String def = "some code";

    List<String> recipients = new ArrayList<String>();

    List<String> types = new ArrayList<String>();
    types.add("groovy");
    methodMock.viewCreateRoute(model, name, "nisse", true, "some description", recipients, "some code", "Name must be specified.");
    methodMockControl.setReturnValue("route_create_groovy");

    replay();

    String result = classUnderTest.createRoute(model, name, author, active,
        description, recipients, def);

    verify();
    assertEquals("route_create_groovy", result);
  }
  
  public void testCreateRoute_noDefinition() {
    String name = "abc";
    String author = "nisse";
    Boolean active = true;
    String description = "some description";
    String def = "";

    List<String> recipients = new ArrayList<String>();

    List<String> types = new ArrayList<String>();
    types.add("groovy");
    methodMock.viewCreateRoute(model, name, author, active, description, recipients, def, "Can not create a groovy rule when script is empty.");
    methodMockControl.setReturnValue("createroute");

    replay();

    String result = classUnderTest.createRoute(model, name, author, active,
        description, recipients, def);

    verify();
    assertEquals("createroute", result);
  }

  public void testCreateRoute_failedCreate() {
    RouteDefinition routedef = new RouteDefinition();
    String name = "MyRoute";
    String author = "nisse";
    Boolean active = true;
    String description = "some description";
    String def = "some code";
    // GroovyRule constructor is protected so mock it
    MockControl ruleControl = MockClassControl.createControl(GroovyRule.class);
    GroovyRule rule = (GroovyRule)ruleControl.getMock();
    
    List<String> recipients = new ArrayList<String>();
    methodMock.createRule(def);
    methodMockControl.setReturnValue(rule);
    manager.create(name, author, active, description, recipients, rule);
    managerControl.setReturnValue(routedef);
    manager.storeDefinition(routedef);
    managerControl.setThrowable(new RuleException("Duplicate name"));
    methodMock.viewCreateRoute(model, name, author, active, description, recipients, def, "Failed to create definition: 'Duplicate name'");
    methodMockControl.setReturnValue("route_create_groovy");
    replay();

    String result = classUnderTest.createRoute(model, name, author, active,
        description, recipients, def);

    verify();
    assertEquals("route_create_groovy", result);
  }

  
  public void testShowRoute_byName() {
    String name = "somename";
    List<String> recipients = new ArrayList<String>();
    
    RouteDefinition routeDefinition = new RouteDefinition();
    routeDefinition.setActive(true);
    routeDefinition.setAuthor("nisse");
    routeDefinition.setDescription("descr");
    routeDefinition.setName(name);
    routeDefinition.setRecipients(recipients);
    routeDefinition.setRule(new GroovyRule() {
      public String getScript() {
        return "def";
      }
      public int getState() {
        return GroovyRule.OK;
      }
    });
    manager.getDefinition(name);
    managerControl.setReturnValue(routeDefinition);
    
    methodMock.viewShowRoute(model, name, "nisse", true, "descr", recipients, "def", null);
    methodMockControl.setReturnValue("route_show_groovy");
    
    replay();

    String result = classUnderTest.showRoute(model, name, null, null, null, null, null, null);
    
    verify();
    assertEquals("route_show_groovy", result);
  }

  public void testShowRoute_byName_nonExisting() {
    String name = "somename";
    manager.getDefinition(name);
    managerControl.setReturnValue(null);
    methodMock.viewShowRoutes(model, "No route named \"somename\"");
    methodMockControl.setReturnValue("routes");
    
    replay();

    String result = classUnderTest.showRoute(model, name, null, null, null, null,null, null);
    
    verify();
    assertEquals("routes", result);
  }

  public void testShowRoute_modify() {
    String name = "somename";
    List<String> newrecipients = new ArrayList<String>();
    
    RouteDefinition routeDefinition = new RouteDefinition();
    
    manager.getDefinition(name);
    managerControl.setReturnValue(routeDefinition);
    methodMock.modifyRoute(model, name, "hugga", false, "new descr", newrecipients, "def");
    methodMockControl.setReturnValue("somedirect");
    
    replay();

    String result = classUnderTest.showRoute(model, name, "hugga", false, "new descr", newrecipients, "def", "Save");
    
    verify();
    assertEquals("somedirect", result);
  }

  public void testShowRoute_delete() {
    String name = "somename";
    RouteDefinition routeDefinition = new RouteDefinition();
    
    manager.getDefinition(name);
    managerControl.setReturnValue(routeDefinition);
    manager.deleteDefinition("somename");
    
    replay();

    String result = classUnderTest.showRoute(model, name, null, null, null, null, null, "Delete");
    
    verify();
    assertEquals("redirect:routes.htm", result);
  }

  public void testShowRoute_delete_failed() {
    String name = "somename";
    RouteDefinition routeDefinition = new RouteDefinition();
    
    manager.getDefinition(name);
    managerControl.setReturnValue(routeDefinition);
    manager.deleteDefinition("somename");
    managerControl.setThrowable(new RuleException());
    methodMock.viewShowRoutes(model, "Failed to delete \"somename\", have you verified that there are no reffering scheduled jobs");
    methodMockControl.setReturnValue("routes");
    
    replay();

    String result = classUnderTest.showRoute(model, name, null, null, null, null, null, "Delete");
    
    verify();
    assertEquals("routes", result);
  }

  public void testViewCreateRoute() {
    List<String> recipients = new ArrayList<String>();
    List<String> adaptors = new ArrayList<String>();
    RouteDefinition routeDefinition = new RouteDefinition();

    adaptorManager.getAdaptorNames();
    adaptorControl.setReturnValue(adaptors);
    model.addAttribute("adaptors", adaptors);
    modelControl.setReturnValue(null);
    manager.create("somename", "someauthor", true, "descr", recipients, null);
    managerControl.setReturnValue(routeDefinition);

    model.addAttribute("route", routeDefinition);
    modelControl.setReturnValue(null);
    model.addAttribute("emessage", null);
    modelControl.setReturnValue(null);
    model.addAttribute("typdef", "def");
    modelControl.setReturnValue(null);

    classUnderTest = new GroovyRoutesController();
    classUnderTest.setManager(manager);
    classUnderTest.setAdaptorManager(adaptorManager);
    
    replay();

    String result = classUnderTest.viewCreateRoute(model, "somename", "someauthor", true, "descr", recipients, "def", null);
    
    verify();
    assertEquals("route_create_groovy", result);
    
  }
  
  public void testViewShowRoutes() {
    List<RouteDefinition> definitions = new ArrayList<RouteDefinition>();
    
    manager.getDefinitions();
    managerControl.setReturnValue(definitions);
    
    model.addAttribute("routes", definitions);
    modelControl.setReturnValue(null);
    
    classUnderTest = new GroovyRoutesController();
    classUnderTest.setManager(manager);
    
    replay();

    String result = classUnderTest.viewShowRoutes(model, null);
    
    verify();
    assertEquals("routes", result);
  }
  
  public void testViewShowRoute() {
    List<String> recipients = new ArrayList<String>();
    List<String> adaptors = new ArrayList<String>();
    RouteDefinition routeDefinition = new RouteDefinition();

    adaptorManager.getAdaptorNames();
    adaptorControl.setReturnValue(adaptors);
    model.addAttribute("adaptors", adaptors);
    modelControl.setReturnValue(null);
    manager.create("somename", "someauthor", true, "descr", recipients, null);
    managerControl.setReturnValue(routeDefinition);
    model.addAttribute("route", routeDefinition);
    modelControl.setReturnValue(null);
    model.addAttribute("definition", "def");
    modelControl.setReturnValue(null);
    model.addAttribute("emessage", null);
    modelControl.setReturnValue(null);
    
    classUnderTest = new GroovyRoutesController();
    classUnderTest.setManager(manager);
    classUnderTest.setAdaptorManager(adaptorManager);
    
    replay();

    String result = classUnderTest.viewShowRoute(model, "somename", "someauthor", true, "descr", recipients, "def", null);
    
    verify();
    assertEquals("route_show_groovy", result);
  }

  public void testModifyRoute() throws Exception {
    String name = "A";
    String author = "B";
    boolean active = false;
    String description = "descr";
    List<String> recipients = new ArrayList<String>();
    String script = "a script";
    // GroovyRule constructor is protected so mock it
    MockControl ruleControl = MockClassControl.createControl(GroovyRule.class);
    GroovyRule rule = (GroovyRule)ruleControl.getMock();
    RouteDefinition definition = new RouteDefinition();
    
    classUnderTest = new GroovyRoutesController() {
      protected GroovyRule createRule(String script) {
        return methodMock.createRule(script);
      }
    };
    classUnderTest.setManager(manager);
    
    methodMock.createRule(script);
    methodMockControl.setReturnValue(rule);
    manager.create(name, author, active, description, recipients, rule);
    managerControl.setReturnValue(definition);
    manager.updateDefinition(definition);
    
    replay();
    
    String result = classUnderTest.modifyRoute(model, name, author, active, description, recipients, script);
    
    verify();
    assertEquals("redirect:routes.htm", result);
  }

  public void testModifyRoute_noScript() throws Exception {
    String name = "A";
    String author = "B";
    boolean active = false;
    String description = "descr";
    List<String> recipients = new ArrayList<String>();
    String script = null;
    
    classUnderTest = new GroovyRoutesController() {
      protected GroovyRule createRule(String script) {
        return methodMock.createRule(script);
      }
      protected String viewShowRoute(Model model,
          String name,
          String author,
          Boolean active,
          String description,
          List<String> recipients,
          String definition,
          String emessage) {
        return methodMock.viewShowRoute(model, name, author, active,
            description, recipients, definition, emessage);
      }
      
    };
    classUnderTest.setManager(manager);
    
    methodMock.viewShowRoute(model, name, author, active, description, recipients, script, "Definition missing.");
    methodMockControl.setReturnValue("someredirect");
    
    replay();
    
    String result = classUnderTest.modifyRoute(model, name, author, active, description, recipients, script);
    
    verify();
    assertEquals("someredirect", result);
  }

  public void testModifyRoute_failedUpdate() throws Exception {
    // GroovyRule constructor is protected so mock it
    MockControl ruleControl = MockClassControl.createControl(GroovyRule.class);
    GroovyRule rule = (GroovyRule)ruleControl.getMock();

    String name = "A";
    String author = "B";
    boolean active = false;
    String description = "descr";
    List<String> recipients = new ArrayList<String>();
    String script = "some def";
    RouteDefinition definition = new RouteDefinition();
     
    classUnderTest = new GroovyRoutesController() {
      protected GroovyRule createRule(String script) {
        return methodMock.createRule(script);
      }
      protected String viewShowRoute(Model model,
          String name,
          String author,
          Boolean active,
          String description,
          List<String> recipients,
          String definition,
          String emessage) {
        return methodMock.viewShowRoute(model, name, author, active,
            description, recipients, definition, emessage);
      }
      
    };
    classUnderTest.setManager(manager);
    methodMock.createRule(script);
    methodMockControl.setReturnValue(rule);
    manager.create(name, author, active, description, recipients, rule);
    managerControl.setReturnValue(definition);
    manager.updateDefinition(definition);
    managerControl.setThrowable(new RuntimeException("Bad..."));
    methodMock.viewShowRoute(model, name, author, active, description, recipients, script, "Failed to update definition: 'Bad...'");
    methodMockControl.setReturnValue("someredirect");
    
    replay();
    
    String result = classUnderTest.modifyRoute(model, name, author, active, description, recipients, script);
    
    verify();
    assertEquals("someredirect", result);
  }
  
  public void testCreateRule() throws Exception {
    MockControl groovyRuleControl = MockClassControl.createControl(GroovyRule.class);
    GroovyRule groovyRule = (GroovyRule)groovyRuleControl.getMock();
    
    classUnderTest = new GroovyRoutesController();
    classUnderTest.setManager(manager);
    
    manager.createRule(GroovyRule.TYPE);
    managerControl.setReturnValue(groovyRule);
    groovyRule.setScript("abc");

    replay();
    
    groovyRuleControl.replay();
    
    GroovyRule result = classUnderTest.createRule("abc");
    
    verify();
    groovyRuleControl.verify();
    assertSame(groovyRule, result);
  }
}
